@@ -51,7 +51,7 @@
 #include "mapping-protocol.h"
 #include "mapping-method.h"
 
-GIOChannel *daemon_ioc = NULL;
+MappingProtocolChannel *daemon_ioc = NULL;
 
 G_LOCK_DEFINE_STATIC (mapping_lock);
 
@@ -220,8 +220,10 @@ monitor_do_iter_unlocked (void)
 		gboolean                  cancelled;
 		GnomeVFSMonitorEventType  event_type;
 
-		if (!event.userdata)
+		if (!event.userdata) {
+			DEBUG_PRINT ("Received event with no userdata, skipping\n");
 			continue;
+		}
 
 		handle = (VirtualFileMonitorHandle *)event.userdata;
 		cancelled = handle->cancelled;
@@ -248,11 +250,21 @@ monitor_do_iter_unlocked (void)
 			} else
 				info_uri = gnome_vfs_uri_append_file_name (handle->uri, event.path);
 
+			DEBUG_PRINT ("Returning event %d path %s handle %p\n",
+				     event_type,
+				     event.path,
+				     handle);
+
 			/* This queues an idle, so there are no reentrancy issues */
 			gnome_vfs_monitor_callback ((GnomeVFSMethodHandle *)handle,
 						    info_uri, 
 						    event_type);
 			gnome_vfs_uri_unref (info_uri);
+		} else {
+			DEBUG_PRINT ("Cancelled event %d path %s handle %p\n",
+				     event_type,
+				     event.path,
+				     handle);
 		}
 	}
 
@@ -1052,7 +1064,7 @@ monitor_setup (void)
 	G_LOCK (mapping_lock);
 
 	if (monitor_watch_id == 0) {
-		monitor_watch_id = g_io_add_watch (daemon_ioc,
+		monitor_watch_id = g_io_add_watch (daemon_ioc->iochannel,
 						   G_IO_IN | G_IO_HUP | G_IO_ERR,
 						   monitor_callback, NULL);
 	}
@@ -1086,7 +1098,7 @@ do_monitor_add (GnomeVFSMethod        *method,
 	monitor->uri = uri;
 	gnome_vfs_uri_ref (uri);
 
-	DEBUG_PRINT ("do_monitor_add: %p\n", monitor);
+	DEBUG_PRINT ("do_monitor_add: %p path: %s\n", monitor, path);
 
 	res = request_op (MAPPING_PROTOCOL_OP_MONITOR_ADD,
 			  monitor->uri->method_string,
@@ -1311,8 +1323,7 @@ vfs_module_init (const char *method_name,
 		}
         }
 
-	daemon_ioc = g_io_channel_unix_new (daemon_fd);
-	g_io_channel_set_close_on_unref (daemon_ioc, TRUE);
+	daemon_ioc = mapping_protocol_channel_new (daemon_fd);
 
 #ifdef DEBUG_ENABLE
 	debug_init ();
@@ -1326,7 +1337,7 @@ vfs_module_shutdown (GnomeVFSMethod *method)
 {
 	monitor_stop_unlocked ();
 
-	g_io_channel_unref (daemon_ioc);
+	mapping_protocol_channel_unref (daemon_ioc);
 
 #ifdef DEBUG_ENABLE
 	fclose (debug_out);
